package com.bwie.house.controller;

import cn.hutool.core.date.DateTime;
import com.alibaba.fastjson.JSONObject;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.House;
import com.bwie.common.domain.Loan;
import com.bwie.common.domain.Rent;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.HouseRequest;
import com.bwie.common.domain.response.Month;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.house.service.HouseService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.github.tobato.fastdfs.domain.fdfs.StorePath;
import com.github.tobato.fastdfs.service.FastFileStorageClient;
import com.mysql.cj.conf.url.FailoverDnsSrvConnectionUrl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.time.Instant;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("/house")
public class HouseController {
    @Autowired
    private HouseService service;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private FastFileStorageClient fastFileStorageClient;

    /**
     * 用户信息
     * @return
     */
    public User userInfo(){
        String token = request.getHeader("token");
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSONObject.parseObject(s, User.class);
        return user;
    }

    /**
     * 上传图片
     * @param myFile
     * @return
     */
    @PostMapping("upload")
    public Result upload(@RequestParam("file") MultipartFile myFile) {
        try {
            StorePath file = fastFileStorageClient.uploadFile(myFile.getInputStream(), myFile.getSize(), myFile.toString().split("\\.")[0], null);
            if (file != null) {
                return Result.success("http://159.75.188.178:8888/" + file.getFullPath());
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return Result.error("上传失败");
    }


    /**
     * 房屋列表
     * @param houseRequest
     * @return
     */
    @PostMapping("showAll")
    public Result showAll(@RequestBody HouseRequest houseRequest){
        PageHelper.startPage(houseRequest.getPageNum(),houseRequest.getPageSize());
        List<HouseRequest> show = service.showAll(houseRequest);
        redisTemplate.opsForValue().set("show",JSONObject.toJSONString(show));
        PageInfo<HouseRequest> pageInfo = new PageInfo<>(show);
        return Result.success(pageInfo);
    }

    /**
     * 房屋信息回显
     * @param houseId
     * @return
     */
    @GetMapping("/byHouseId/{houseId}")
    public Result byHouseId(@PathVariable Integer houseId){
        House house = service.byHouseId(houseId);
        return Result.success(house);
    }

    /**
     * 租房贷款审批记录
     * @return
     */
    @GetMapping("showLoan")
    public Result showLoan(){
        List<Loan> show = service.showLoan();
        return Result.success(show);
    }

    /**
     * 房屋信息回显
     * @param loanId
     * @return
     */
    @GetMapping("/byLoanId/{loanId}")
    public Result byLoanId(@PathVariable Integer loanId){
        Loan loan = service.byLoanId(loanId);
        return Result.success(loan);
    }

    /**
     * 合同期限下拉框
     * @return
     */
    @GetMapping("downMonth")
    public Result downMonth(){
        List<Month> down = service.downMonth();
        return Result.success(down);
    }

    /**
     * 租房
     * @param rent
     * @return
     */
    @PostMapping("addHouse")
    public Result addHouse(@RequestBody Rent rent){
        Integer add = service.addHouse(rent);
        return add>0?Result.success(rent):Result.error("租房失败");
    }
}
