package com.bwie.house.service.impl;

import cn.hutool.core.date.DateTime;
import com.alibaba.fastjson.JSONObject;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.House;
import com.bwie.common.domain.Loan;
import com.bwie.common.domain.Rent;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.HouseRequest;
import com.bwie.common.domain.response.Month;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.house.mapper.HouseMapper;
import com.bwie.house.service.HouseService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.time.Instant;
import java.util.List;
import java.util.UUID;

@Service
public class HouseServiceImpl implements HouseService {
    @Autowired
    private HouseMapper houseMapper;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Override
    public List<HouseRequest> showAll(HouseRequest houseRequest) {
        return houseMapper.showAll(houseRequest);
    }

    @Override
    public List<Loan> showLoan() {
        return houseMapper.showLoan();
    }

    @Override
    public Loan byLoanId(Integer loanId) {
        return houseMapper.byLoanId(loanId);
    }

    @Override
    public House byHouseId(Integer houseId) {
        return houseMapper.byHouseId(houseId);
    }

    @Override
    public List<Month> downMonth() {
        return houseMapper.downMonth();
    }

    @Override
    public Integer addHouse(Rent rent) {
        String token = request.getHeader("token");
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSONObject.parseObject(s, User.class);
        DateTime dateTime = new DateTime();
        if (user.getUserStatus() == 0) {
            if (dateTime.getTime() > DateTime.from(Instant.parse(user.getUserDate())).getTime()) {
                House house = new House();
                List<House> list = houseMapper.findId(house.getHouseId());
                for (House house1 : list) {
                    rent.setHouseId(house1.getHouseId());
                    Integer add = houseMapper.addHouse(rent);
                    //租房成功给用户添加一次租房信息
                    if (add > 0) {
                        houseMapper.updSize(user.getUserId());
                        List<Month> find = houseMapper.findMonth(rent.getMonthId());
                    }
                }
            }
        }else{
            Result.error("您已经存在租房记录，并且未到期，不可以再次租房");
        }
        return houseMapper.addHouse(rent);
    }

    //使用定时任务每30秒自动审批用户贷款申请
    @Scheduled(cron = "0/30 * * * * ?")
    public void Apply(){

        //审批实际上就是根据用户的信用分来进行计算，每100信用分可享受月供1000元额度。  根据贷款总价/合约周期计算月供额度是否足够，足够则审批通过，不够则审批不通过
        for (Loan loan : houseMapper.showLoan()) {
            List<Loan> find= houseMapper.findLoanId(loan.getLoanId());
            for (Loan loan1 : find) {
                //计算用户信用额度
                Long sum= (long) (Integer.valueOf(loan1.getLoanAmount())/loan1.getLoanWeek());
                Integer oK= Math.toIntExact(Integer.valueOf(loan1.getLoanAmount()) / sum * 10);
                //不足够
                if (oK > 0) {
                    //审核驳回需要给申请人发送通知信息，通知消息格式自定，消息方式自定
                    houseMapper.updStatus(loan1.getLoanId(),loan1.getLoanRemark());
                    //获取用户信息
                    String token = request.getHeader("token");
                    String userKey = JwtUtils.getUserKey(token);
                    String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
                    User user = JSONObject.parseObject(s, User.class);
                    rabbitTemplate.convertAndSend("sendMessage",user.getUserphone(),message -> {
                        message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
                        return message;
                    });
                }
                //足够
                if (oK < 0) {
                    houseMapper.updStatusOk(loan1.getLoanId());
                }
            }
        }
    }
}
