package com.bwie.rabbitmq.consumer;

import com.alibaba.fastjson.JSONObject;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.TelSmsUtils;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.HashMap;

@Component
@Log4j2
public class MqConsumer {
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private HttpServletRequest request;

    @RabbitListener(queuesToDeclare = @Queue("sendMessage"))
    public void sendMessage(Message message, Channel channel){
        String token = request.getHeader("token");
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSONObject.parseObject(s, User.class);
        String messageId = message.getMessageProperties().getMessageId();
        Long add = redisTemplate.opsForSet().add(messageId, "sendMessage");
        if (add == 1) {
            log.info("MQ发送成功");
            HashMap<String, String> map = new HashMap<>();
            map.put(user.getUserphone(),"审核被驳回 信用不够 谢谢配合");
            TelSmsUtils.sendSms(user.getUserphone(),map);
        }
        try {
            channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
        } catch (IOException e) {
            try {
                channel.basicAck(message.getMessageProperties().getDeliveryTag(),true);
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }
    }
}
